#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'puppet/provider/equallogic'
require 'puppet/util/network_device/equallogic/device'
require 'fixtures/transport/transport_mock'


describe Puppet::Type.type(:equallogic_volume).provider(:equallogic_volume) do

	before :each do
        @transport = Transport_mock.new.transport
        described_class.stubs(:suitable?).returns true
        Puppet::Type.type(:equallogic_volume).stubs(:defaultprovider).returns described_class
    end
      
    let :create_volume do
        Puppet::Type.type(:equallogic_volume).new(
			:name              => 'NewVolume1',
			:poolname          => 'default',
			:ensure        	   => 'present',
			:initsize          => '1m',
			:state       	   => :online,
			:multihostaccess   => 'disable',
			:snapreserve       => '20%',
			:thinprovision     => 'enable',
			:thinminreserve    => '20%',
			:thingrowthwarn    => '20%',
			:thingrowthmax     => '20%',
			:thinwarnsoftthres => '20%',
			:thinwarnhardthres => '20%',
        )
    end
	
	let :createvolume_without_tprovision do
        Puppet::Type.type(:equallogic_volume).new(
			:name              => 'NewVolume1',
			:poolname          => 'default',
			:ensure        	   => 'present',
			:initsize          => '1m',
			:state       	   => :online,
			:multihostaccess   => 'disable',
			:snapreserve       => '20%',
			:thinprovision     => 'disable',
        )
    end
	
	let :destroy_volume do
		Puppet::Type.type(:equallogic_volume).new(
			:name          	   => 'NewVolume1',
			:ensure        	   => 'absent',
		)
	end
	
	let :volume_state do
		Puppet::Type.type(:equallogic_volume).new(
			:name          	   => 'NewVolume1',
			:ensure        	   => 'present',
			:state 			   => 'offline',
		)
	end
	
	let :multihostaccess_volume do
		Puppet::Type.type(:equallogic_volume).new(
			:name          	   => 'NewVolume1',
			:ensure        	   => 'present',
			:multihostaccess   => 'disable'
		)
	end
  
    let :provider do
        described_class.new()
    end
  
      
	context "when create for a given volume is run  " do
		it "should have create method defined for create from volume" do
			create_volume.provider.class.instance_method(:create).should_not == nil
		end
		it "should have destroy method defined for  create from volume" do
			create_volume.provider.class.instance_method(:destroy).should_not == nil
		end
		it "should have exists? method defined for  create from volume" do
			create_volume.provider.class.instance_method(:exists?).should_not == nil
		end
	end
     
	describe "when asking exists?" do
		it "should return true if resource/volume is present" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.stubs(:exists?).once.returns(true)
			create_volume.provider.exists?.should eq(true)
		end
		it "should return false if resource/volume is absent" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.stubs(:exists?).once.returns(false)
			create_volume.provider.exists?.should eq(false)      
		end
	end
  
	describe "for create equallogic volume." do
		it "when creating volume with thinprovision valid credentials" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.stubs(:create_volume_with_thinprovision).once.returns("Volume creation succeeded.")
			create_volume.provider.create
		end
		it "when creating volume with thin provision if Invalid credentials" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.expects(:create_volume_with_thinprovision).once.returns("% Error")
			expect {create_volume.provider.create}.to raise_error(Puppet::Error)
		end
		it "when creating volume with thinprovision  if Not correct Parameters" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.expects(:create_volume_with_thinprovision).once.returns("Error: Only ASCII")
			expect {create_volume.provider.create}.to raise_error(Puppet::Error)
		end
		it "when creating volume without thinprovision valid credentials" do
			createvolume_without_tprovision.provider.stubs(:transport).returns @transport
			createvolume_without_tprovision.provider.stubs(:create_volume_without_thinprovision).once.returns("Volume creation succeeded.")
			createvolume_without_tprovision.provider.create
		end
		it "when creating volume without thinprovision if Invalid credentials" do
			createvolume_without_tprovision.provider.stubs(:transport).returns @transport
			createvolume_without_tprovision.provider.expects(:create_volume_without_thinprovision).once.returns("% Error")
			expect {createvolume_without_tprovision.provider.create}.to raise_error(Puppet::Error)
		end
		it "when creating volume without thinprovision  if Not correct Parameters" do
			createvolume_without_tprovision.provider.stubs(:transport).returns @transport
			createvolume_without_tprovision.provider.expects(:create_volume_without_thinprovision).once.returns("Error: Only ASCII")
			expect {createvolume_without_tprovision.provider.create}.to raise_error(Puppet::Error)
		end
		it "when creating volume set multihome access fails" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.expects(:set_multihost_access).once.returns("% Failed to share the volume")
			expect {create_volume.provider.create}.to raise_error(Puppet::Error)
		end
	end
  
	describe "for destroy equallogic volume." do
		it "when creating volume with thinprovision valid credentials" do
			destroy_volume.provider.stubs(:transport).returns @transport
			destroy_volume.provider.stubs(:set_volume_offline).once.returns('')
			destroy_volume.provider.stubs(:destroy_volume).once.returns("Volume deletion succeeded.")
			destroy_volume.provider.destroy
		end
		it "when destroying volume set volume offline fails" do
			destroy_volume.provider.stubs(:transport).returns @transport
			destroy_volume.provider.expects(:set_volume_offline).once.returns("% Error")
			expect {destroy_volume.provider.destroy}.to raise_error(Puppet::Error)
		end
		it "when destroying volume fails" do
			destroy_volume.provider.stubs(:transport).returns @transport
			destroy_volume.provider.expects(:destroy_volume).once.returns("% Error")
			expect {destroy_volume.provider.destroy}.to raise_error(Puppet::Error)
		end
		it "when destroying equallogic volume with ensure == absent." do
			destroy_volume.provider.stubs(:transport).returns @transport
			destroy_volume.provider.stubs(:destroy).returns(:absent)
			destroy_volume.provider.destroy.should eq(:absent)        
		end
	end
	
	describe "for change of equallogic volume state." do
		it "when set volume state to online" do
			volume_state.provider.stubs(:transport).returns @transport
			volume_state.provider.stubs(:set_volume_online).returns('')
			volume_state.provider.state='online'  
		end
		it "when set volume state to offline" do
			volume_state.provider.stubs(:transport).returns @transport
			volume_state.provider.stubs(:set_volume_offline).once.returns('')
			volume_state.provider.state='offline'
		end
		it "when destroying volume set volume online fails" do
			volume_state.provider.stubs(:transport).returns @transport
			volume_state.provider.expects(:set_volume_online).once.returns("% Error")
			expect {volume_state.provider.state='online'}.to raise_error(Puppet::Error)
		end
		it "when destroying volume set volume offline fails" do
			volume_state.provider.stubs(:transport).returns @transport
			volume_state.provider.expects(:set_volume_offline).once.returns("% Error")
			expect {volume_state.provider.state='offline'}.to raise_error(Puppet::Error)
		end
	end
	
	describe "for change the state of equallogic volume multihostaccess." do
		it "when changing multihostaccess to disable state" do
			multihostaccess_volume.provider.stubs(:transport).returns @transport
			multihostaccess_volume.provider.stubs(:set_multihost_access).once.returns('')
			multihostaccess_volume.provider.multihostaccess='disable'
		end
		it "when changing multihostaccess to enable state" do
			multihostaccess_volume.provider.stubs(:transport).returns @transport
			multihostaccess_volume.provider.stubs(:set_multihost_access).once.returns('')
			multihostaccess_volume.provider.multihostaccess='enable'
		end
		it "when changing multihostaccess to disable fails - invalid credentials" do
			multihostaccess_volume.provider.stubs(:transport).returns @transport
			multihostaccess_volume.provider.expects(:set_multihost_access).once.returns("% Error")
			expect {multihostaccess_volume.provider.multihostaccess='disable'}.to raise_error(Puppet::Error)
		end
		it "when changing multihostaccess to enable fails - invalid credentials" do
			multihostaccess_volume.provider.stubs(:transport).returns @transport
			multihostaccess_volume.provider.expects(:set_multihost_access).once.returns("% Error")
			expect {multihostaccess_volume.provider.multihostaccess='enable'}.to raise_error(Puppet::Error)
		end
	end
	
	describe "when flush" do
		it "create volume flush should be successful" do
			create_volume.provider.stubs(:transport).returns @transport
			create_volume.provider.set(:ensure => :present)
			create_volume.provider.flush
		end
		it "destroy volume flush should be successful" do
			destroy_volume.provider.stubs(:transport).returns @transport
			destroy_volume.provider.set(:ensure => :absent)
			destroy_volume.provider.stubs(:flush).returns(true)
		end
	end
end
