#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'

describe Puppet::Type.type(:equallogic_volume) do

	let(:title) { 'equallogic_volume' }

	context 'should compile with given test params' do
		let(:params) { {
			:name   => 'DemoAlias',
			:poolname    => 'default',
			:initsize     => '1m',
			:thinprovision    => 'enable',
			:snapreserve    => '20%',
			:thinminreserve   => '20%',
			:thingrowthwarn   => '20%',
			:thingrowthmax    => '20%',
			:thinwarnsoftthres  => '20%',
			:thinwarnhardthres  => '20%'
		}}
		it do
			expect {
			should compile
			}
		end

	end
  
	let :resource do
		described_class.new(
			:name          		=> 'NewVolume1',
			:poolname         => 'default',
			:ensure        		=> 'present',
			:state				=> 'online',
			:multihostaccess 	=> 'disable',
			:initsize			=> '1m',
			:thinprovision		=> 'enable',
			:snapreserve		=> '20%',
			:thinminreserve		=> '20%',
			:thingrowthwarn		=> '30%',
			:thingrowthmax		=> '40%',
			:thinwarnsoftthres	=> '50%',
			:thinwarnhardthres	=> '60%',
		)
	end

	it "should have name as its keyattribute" do
		described_class.key_attributes.should == [:name]
	end

	describe "when validating attributes" do
		[:ensure,:state, :multihostaccess].each do |property|
			it "should hava a #{property} property" do
				described_class.attrtype(property).should == :property
			end
		end
		[:name, :poolname, :initsize, :snapreserve, :thinminreserve, :thingrowthwarn, :thingrowthmax, :thinwarnsoftthres, :thinwarnhardthres, :thinprovision].each do |param|
			it "should have a #{param} param" do
				described_class.attrtype(param).should == :param
			end
		end
	end

  
	describe "when validating values" do
  
		describe "for name" do
			it "should allow a valid volume name where ensure is present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:name].should == 'newvolOne'
			end
			it "should allow a valid volume name where ensure is absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:name].should == 'newvolTwo'
			end
		end
		
    describe "for pool name" do
      it "should allow a valid volume name where ensure is present" do
        described_class.new( :name => 'newvolOne',:poolname => 'default', :ensure => 'present')[:poolname].should == 'default'
      end
      it "should allow a valid pool name where ensure is absent" do
        described_class.new(:name => 'newvolOne',:poolname => 'default', :ensure => 'absent')[:poolname].should == 'default'
      end
    end

		describe "for ensure" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:ensure].should == :present
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:ensure].should == :absent
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'foo') }.to raise_error Puppet::Error
			end
		end
	
		describe "for state" do
			it "should allow online" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :state => :online)[:state].should == :online
			end
			it "should allow offline" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :state => :offline)[:state].should == :offline
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'present', :state => 'invalid') }.to raise_error Puppet::Error
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'absent', :state => 'invalid') }.to raise_error Puppet::Error
			end
		end
	
		describe "for multihostaccess" do
			it "should allow enable state" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :multihostaccess => :enable)[:multihostaccess].should == :enable
			end
			it "should allow disable state" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :multihostaccess => :disable)[:multihostaccess].should == :disable
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'present', :multihostaccess => 'invalid') }.to raise_error Puppet::Error
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'absent', :multihostaccess => '!!@@##') }.to raise_error Puppet::Error
			end
		end
	
		describe "for initsize" do
			it "should allow 1t" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :initsize => '1t')[:initsize].should == '1t'
			end
			it "should allow 1g" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :initsize => '1g')[:initsize].should == '1g'
			end
			it "should allow 1m" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :initsize => '1m')[:initsize].should == '1m'
			end
			it "should allow 1t" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :initsize => '1t')[:initsize].should == '1t'
			end
			it "should allow 1g" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :initsize => '1g')[:initsize].should == '1g'
			end
			it "should allow 1m" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :initsize => '1m')[:initsize].should == '1m'
			end
			it "should not allow something else invalid alphanumericals" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'present', :initsize => 'invalid') }.to raise_error Puppet::ResourceError
			end
			it "should not allow something else invalid special characters" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'absent', :initsize => '!!@@##') }.to raise_error Puppet::ResourceError
			end
		end
	
		describe "for thinprovision" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present', :thinprovision => :enable)[:thinprovision].should == :enable
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent', :thinprovision => :disable)[:thinprovision].should == :disable
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'present', :thinprovision => 'invalid') }.to raise_error Puppet::ResourceError
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'absent', :thinprovision => '!!@@##') }.to raise_error Puppet::ResourceError
			end
		end
      
		[:snapreserve, :thinminreserve, :thingrowthwarn, :thingrowthmax, :thinwarnsoftthres, :thinwarnhardthres].each do |limit_property|
			describe "for #{limit_property}" do
				it "should allow value specified in 0%" do
					described_class.new(:name => 'newvolFive', :ensure => 'present', limit_property => '0%')[limit_property].should == '0%'
				end
				it "should allow value specified in between 0% and 100%" do
					described_class.new(:name => 'bob', :ensure => 'present', limit_property => '10%')[limit_property].should == '10%'
				end
				it "should allow value specified in 100%" do
					described_class.new(:name => 'bob', :ensure => 'present', limit_property => '100%')[limit_property].should == '100%'
				end
				it "should not allow negative values like -10%, ensure is present" do
					expect { described_class.new(:name => 'bob', :ensure => 'present', limit_property => '-10%') }.to raise_error Puppet::ResourceError
				end
				it "should not allow values morethan 100%, ensure is present" do
					expect { described_class.new(:name => 'bob', :ensure => 'present', limit_property => '110%') }.to raise_error Puppet::ResourceError
				end
				it "should allow value specified in 0%" do
					described_class.new(:name => 'newvolFive', :ensure => 'absent', limit_property => '0%')[limit_property].should == '0%'
				end
				it "should allow value specified in between 0% and 100%" do
					described_class.new(:name => 'bob', :ensure => 'absent', limit_property => '10%')[limit_property].should == '10%'
				end
				it "should allow value specified in 100%" do
					described_class.new(:name => 'bob', :ensure => 'absent', limit_property => '100%')[limit_property].should == '100%'
				end
				it "should not allow negative values like -10%, ensure is absent" do
					expect { described_class.new(:name => 'bob', :ensure => 'absent', limit_property => '-10%') }.to raise_error Puppet::ResourceError
				end
				it "should not allow values morethan 100%, ensure is absent" do
					expect { described_class.new(:name => 'bob', :ensure => 'absent', limit_property => '110%') }.to raise_error Puppet::ResourceError
				end
			end
		end
  end	
end
