#! /usr/bin/env ruby

require 'spec_helper'
require 'puppet/util/network_device/equallogic/device'
require 'puppet/util/network_device/equallogic/facts'
require 'puppet/util/network_device/transport_equallogic'
require 'yaml'
require 'uri'
require 'fixtures/transport/transport_mock'

describe Puppet::Util::NetworkDevice::Equallogic::Device do

	before :each do
		@transport = Transport_mock.new.transport
	end

	device_url = 'https://mockeqladmin:mockeqladmin@192.168.1.100?community_string=snmpstring'
	let :device_class do
		described_class.new(device_url)
	end
  
	describe "connection and facts - when connecting to a new device" do

		it "should reject a single hostname" do
			deviceobj = described_class.new('eql.example.com')
			urlobj = deviceobj.url
			expect { urlobj.scheme }.to raise_error  ArgumentError
		end

		it "should reject a missing username" do
			deviceobj = described_class.new('https://eql.example.com')
			urlobj = deviceobj.url
			expect { urlobj.user }.to raise_error ArgumentError
		end

		it "should reject a missing password" do
			deviceobj = described_class.new('https://root@eql.example.com')
			urlobj = deviceobj.url
			expect { urlobj.password }.to raise_error ArgumentError
		end

		it "should not accept plain http connections" do
			deviceobj = described_class.new('http://root@eql.example.com')
			urlobj = deviceobj.url
			expect { urlobj.scheme }.to raise_error ArgumentError
		end
	
		it "should reject a invalid url" do
			described_class.stubs(:transport).returns @transport
			described_class.stubs(:checkConnection).returns("Group Information")
			described_class
		end
	
		it "should give connection error - if url not accessible" do
			deviceobj = described_class.new('https://mockeqladmin:mockeqladmin@192.168.1.100')
			deviceobj.stubs(:checkConnection).once.returns("Error")
			expect {deviceobj}.to raise_error(Puppet::Error)
		end
	
		it "should get valid error message if facts retrive fails" do
			described_class.stubs(:transport).returns @transport
			described_class.stubs(:facts).returns({})
			described_class.facts.should eq({})
		end
	
		it "should get valid error message if facts retrive fails" do
			described_class.stubs(:transport).returns @transport
			described_class.stubs(:facts).returns("Error")
			described_class.facts
		end
	
		context "#parse_snmp" do
			it "should get snmpstring from url - with query" do
				device_url = 'https://mockeqladmin:mockeqladmin@192.168.1.100?community_string=snmpstring'
				@url = URI.parse(device_url)
				described_class.stubs(:transport).returns @transport
				described_class.stubs(:parse_snmp).with(@url.query).returns('community_string=snmpstring')
				described_class.parse_snmp(@url.query).should eq('community_string=snmpstring')
			end
			it "should get snmpstring from url - no query" do
				device_url = 'https://mockeqladmin:mockeqladmin@192.168.1.100'
				@url = URI.parse(device_url)
				described_class.stubs(:transport).returns @transport
				described_class.stubs(:parse_snmp).with(@url.query)
				described_class.parse_snmp(@url.query).should eq(nil)
			end
		end

		context "#facts" do
			it "should retrive facts from equllogic" do
				described_class.stubs(:transport).returns @transport
				described_class.stubs(:facts).once.returns({})
				described_class.facts.should eq({})
			end
		end
  end
end
